import json
import os

import xlrd

from tool.config import Config


def params_to_dict(s):
    d={}
    try:
        json.loads(s)
    except:
        pass
    else:
        return d
    # if "=" not in d:
    #     return {}
    # else:
    kvs=s.split('&')
    for kv in kvs:
        d[kv.split('=')[0]]=kv.split('=')[1]
    return d



def headers_to_dict(s):
    d={}
    s=s.strip()
    try:
        json.loads(s)
    except:
        pass
    else:
        return d

    dict_list=s.split('\n')
    for kvs in dict_list:
        kvs=kvs.strip()
        k,v=kvs.split(': ')
        d[k]=v
    return d



def read_excel(path):
    root_path=Config().get_rootpath()
    excel=xlrd.open_workbook(os.path.join(root_path,path))
    sheets=excel.sheets()    #读取所以sheet
    ids=[]
    case_list=[]
    for sheet in sheets:
        rows=sheet.nrows
        if rows<1:
            continue
        rows_1=sheet.row_values(0)   #读取第一行数据
        for i in range(1,rows):
            row=sheet.row_values(i)
            d={}
            for n in range(len(row)):
                d[rows_1[n]] = None if row[n]=="" else row[n]
            if d['is_run']=='否':
                continue
            ids.append(d['title'])
            case_list.append(d)
    return ids,case_list




def scan_excel(root_path,excel_list):        #扫描项目下的所有Excel文件
    files_list=os.listdir(root_path)
    for f in files_list:
        file=os.path.join(root_path,f)
        if os.path.isdir(file) and f not in ['.idea', '.pytest_cache', '__pycache__']:
            scan_excel(file,excel_list)
        elif os.path.isfile(file) and (file.endswith(".xls")) or (file.endswith(".xlsx")):        #判断是否是Excel文件
            excel_list.append(file)
        else:
            pass

def get_cases():
    root_path=Config().get_rootpath()
    excel_list=[]
    scan_excel(root_path,excel_list)
    # 批量读取Excel文件里面的数据
    ids=[]
    cases=[]
    for e in excel_list:
        titles,case_list=read_excel(e)
        ids.extend(titles)
        cases.extend(case_list)
    return ids,cases






import random



def random_str(send_request,chars,lenth):
    return "".join(random.choices(chars,k=int(lenth)))


def random_int(send_request,star,end):
    return random.randint(int(star),int(end))

def phone(send_request):
    return "".join('18949458978')


import json
import re

from tool.key_map import key_mps


class KeyDriver():
    def __init__(self,send_request,value):
        self.send_request=send_request
        self.value=self.to_str(value)
        self.key_replace()



    def to_str(self,value):
        if isinstance(value,dict) or isinstance(value,list):
            value=json.dumps(value,ensure_ascii=False)
        else:
            value=str(value)
        return value



    def paser(self,value):
        '''
        返回字符串中的所有关键字
        $__PHONE()$
        $__RANDOM_STR(abcdefg,10)$
        $__RANDOM_INT$(6,10)$
        $__RANDOM_STR(abcdefg,$__RANDOM_INT$(6,10)$)$
        :return:
        '''
        star=-1
        flag=0
        lenth=len(value)
        keys=[]
        for key in range(lenth):
            if value[key]=='$':
                if key+2<lenth and value[key:key+3]=='$__':
                    # print(f'{key}起始关键字')
                    flag-=1
                    if star<0:
                        star=key
                elif key - 1>0 and value[key-1 : key+1]==")$":
                    # print(f'{key}为结束字符')
                    flag+=1
                    if flag==0:
                        # print(value[star:key+1])
                        keys.append(value[star:key+1])
                        star=-1
        return keys


    def key_run(self,key):
        if "$__" in key[3:-1]:
            in_keys = self.paser(key[3:-1])
            for i_key in in_keys:
                res=self.key_run(i_key)
                key=key.replace(i_key,res)
        key_name=key[3:key.find("(")]
        r=re.compile(r"\((.*?)\)")
        key_args=r.findall(key)[0]
        if key_args=="":
            res=key_mps[key_name](self.send_request)
        else:
            res=key_mps[key_name](self.send_request,*key_args.split(","))
        return self.to_str(res) if res else None


    def key_replace(self):
        keys=self.paser(self.value)
        for key in keys:
            res=self.key_run(key)
            if res:
                self.value = self.value.replace(key, self.to_str(res))
        return self.value


    def __str__(self):
        return self.value








import logging



#获取日志记录器
import os
from logging.handlers import TimedRotatingFileHandler

logger=logging.getLogger("自动化框架搭建")
logger.setLevel(level=logging.DEBUG)  # 设置日志记录器的最低级别
file_path=os.path.join(os.getcwd(),"logs/") #生成文件的位置
if not os.path.exists(file_path):  #判断文件是否存在，如果不存在就创建一个
    os.mkdir(file_path)

#debug日志
#创建文件hander
# 创建一个按大小自动切割日志的日志处理器。 root_path + 'api.log' api日志路径，maxBytes 设置日志切割大小，单位时byte
# backupCount 设置备份文件的最大个数，encoding 设置日志文件编码格式
hander=TimedRotatingFileHandler(file_path+"api.log",when="d",interval=1,backupCount=15,encoding="utf-8")
hander.setLevel(logging.DEBUG)
# 创建一个日志格式化器
formatter=logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')  # 日志格式化期和日志处理器绑定
hander.setFormatter(formatter)
logger.addHandler(hander)


#报错日志
hander2=TimedRotatingFileHandler(file_path+"erro.log",when="d",interval=1,backupCount=15,encoding="utf-8")
hander2.setLevel(logging.ERROR)
formatter=logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
hander2.setFormatter(formatter)
logger.addHandler(hander2)


#正常处理日志
hander3=logging.StreamHandler()
hander3.setLevel(logging.DEBUG)
formatter=logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
hander3.setFormatter(formatter)
logger.addHandler(hander3)


def info(msg):
    logger.info(msg)

def erro(msg):
    logger.error(msg)

def debug(msg):
    logger.debug(msg)


import json
import os

import requests

from tool.config import Config
from tool.tools import params_to_dict, headers_to_dict


class BaseConfig():
    __config=None
    @property
    def config(self):
        if self.__config is None:
            __class__.__config=Config()
        return self.__config

class Configdes(BaseConfig):
    def __set__(self, instance, value):
        pass
    def __get__(self, instance, owner):
        return self.config








class BaseRequests():
    method=None
    name=None
    __url=None
    __json=None
    __data=None
    __params=None
    __headers=None
    __files=None
    config=Configdes()


    @property   #把2一个方法定义成一个属性的装饰器
    def url(self):
        return self.config.get_ip(self.name) + self.__url
    @url.setter
    def url(self,value):
        self.__url=value


    @property
    def json(self):
        return self.__json
    @json.setter
    def json(self,value):
        try:
            self.__json=json.loads(value)
        except:
            self.__json=value




    @property
    def data(self):
        return self.__data
    @data.setter
    def data(self,value):
        try:
            self.__data=json.loads(value)
        except:
            self.__data=value


    @property
    def params(self):
        return self.__params
    @params.setter
    def params(self,value):
        if isinstance(value,dict):
            self.__params=value
        elif isinstance(value,str):
            self.__params= params_to_dict(value)


    @property
    def headers(self):
        return self.__headers
    @headers.setter
    def headers(self,value):
        if not value:
            return
        try:
            self.__headers=json.loads(value)
        except:
            self.__headers=headers_to_dict(value)







    @property
    def files(self):
        if not self.__files:
            return
        try:
            self.__files = json.loads(self.__files)
        except:
            pass
        if isinstance(self.__files, dict):
            for i in self.__files:
                if isinstance(self.__files[i], str):
                    self.__files[i] = open(os.path.join(self.config.get_rootpath(), self.__files[i]), "rb")
        return self.__files

    @files.setter
    def files(self, value):
        self.__files = value




from tool.jsonpath_tool import json_excutor, get_var, set_var, assert_contains, assert_json
from tool.key_tools import random_str, random_int, phone

key_mps={
    "RANDOM_STR": random_str,
    "RANDOM_INT": random_int,
    "PHONE": phone,
    "JSON_EXCUTOR": json_excutor,
    "GET_VAR": get_var,
    "SET_VAR": set_var,
    "ASSERT_CONTAINS": assert_contains,
    "ASSERT_JSON": assert_json,
}


import json

import allure
import requests

from common.baserequest import BaseRequests
from common.baseresponse import BaseResponse
from tool.key_driver import KeyDriver


class SendRequest():
    dict_var={}

    def __init__(self,case):
        self.request=BaseRequests()
        self.request_data={}
        self.allure_data = {"feature": case["name"]}
        for key in case:
             if key in["before"]:
                 self.before=case["before"]
             elif key in["assert"]:
                self.assert_data=case["assert"]
             elif key in ["back"]:
                self.back=case["back"]
             elif key in ["story", "title"]:
                self.allure_data[key] = case[key]
             elif key in ["name","url","method","params","data","json","headers","files"]:
                 self.request_data[key]=case[key]
        for key in self.allure_data:
            getattr(allure.dynamic,key)(self.allure_data[key])




    def send(self):
        #第一步执行前置操作
        KeyDriver(self,self.before)
        #第二步发送请求
        self.request_data=json.loads(str(KeyDriver(self,self.request_data)))
        for key in self.request_data:
            setattr(self.request,key,self.request_data[key])


        resp=requests.request(
            method= self.request.method,
            url= self.request.url,
            params= self.request.params,
            data= self.request.data,
            json= self.request.json,
            headers= self.request.headers,
            files= self.request.files
        )
        self.response=BaseResponse(resp)
        # 第三部执行后置动作
        KeyDriver(self,self.back)

        KeyDriver(self,self.assert_data)


from tool import log


class BaseResponse():
    def __init__(self,response):
        self.response=response
        self.print_log()

    @property
    def status_code(self):
        return self.response.status_code


    @property
    def res_text(self):
        return self.response.text



    @property
    def res_json(self):
        return self.response.json()


    def print_log(self):
        log.info(f"""---------------------响应报文--------------------------
{self.status_code}
{self.res_text}""")





# json提取器
import random

from tool import log
import jsonpath

def json_excutor(send_request,var_name,index,*json_path):
    json_path=",".join(json_path)
    if not hasattr(send_request,'response'):
        log.erro(f"$__JSON_EXCUTOR({index},{jsonpath})$关键字只能用于后置或者断言中")
        return
    response=None
    try:
        response=send_request.response.res_json
    except:
        log.erro(f"$__JSON_EXCUTOR({index},{jsonpath})$关键字只能用于json格式")
        raise
    json_path = json_path.replace('"', "'")
    res_list=jsonpath.jsonpath(response,json_path)
    if res_list:
        index=int(index)
        if index>0:
            res=res_list[index-1]
        else:
            res=random.choice(res_list)

        send_request.dict_var[var_name]=res
        # log.debug(f"变量值{var_name}:{res}")


def get_var(send_request,var_name):       #取变量
    vars=send_request.dict_var
    if var_name in vars:
        return vars[var_name]



def set_var(send_request,var_name,value):   #存变量
    send_request.dict_var[var_name]=value





def assert_contains(send_request,*hope):          #包含断言
    hope=",".join(hope)
    if not hasattr(send_request,'response'):
        log.erro(f"$__assert_contains{hope}关键字，只能用于back或assert当中")
        return
    body_text=send_request.response.res_text
    try:
        assert hope in body_text
    except:
        log.erro(f"断言失败，响应正文中不包含：{hope}")
        raise



def assert_json(send_request,hope,*json_path):    #json断言
    json_path=",".join(json_path)
    if not hasattr(send_request,'response'):
        log.erro(f"$__assert_contains{hope}关键字，只能用于back或assert当中")
        return
    response=None
    try:
        response=send_request.response.res_json
    except:
        log.erro(f"$__assert_json{hope},{json_path}$关键字，只能用于响应数据为json格式")
        raise
    json_path=json_path.replace('"',"'")
    res=jsonpath.jsonpath(response,json_path)
    log.debug(json_path)
    try:
        if res :
            if hope.isdigit():
                assert res[0]==int(hope)
            else:
                assert res[0]==hope
        else:
            assert False
    except:
        log.erro(f"断言失败，json_path:{json_path}对应数据{res}不存在，或与预期结果不一致")
        raise





import configparser
import os

config=configparser.ConfigParser()
file_path=os.path.join(os.getcwd(),'config.ini')
config.read(file_path,encoding='utf-8')



class Config():
    def __init__(self):
        # 获取环境号
        # self.env=os.environ.get('env')
        self.env='test'

    def get_ip(self,name):
        section=f"{self.env}_ip"
        return self.get_option(section,name)




    def section_is_exist(self,section):
        return section in config.sections()


    def get_option(self,section,option):
        try:
            return config.get(section,option)
        except:
            assert False,'获取不到配置信息'

    def get_rootpath(self):
        root_path=self.get_option("root_path","path")
        if root_path =="":
            root_path=os.path.dirname(file_path)
        return root_path





from setuptools import setup, find_packages

setup(
    name='xiaoheauto',  # 包名    pip install 包名
    version='1.0.0',  # 版本
    keywords=["api","auto"],
    description="test packge auto",  # 包简介
    long_description='Interface automation framework',  # 读取文件中介绍包的详细内容
    author='hezhibin',  # 作者
    author_email='2587071020@qq.com',  # 作者邮件
    license='MIT License',  # 协议
    url='https://gitee.com/hestudyit/autoapi.git',  # github或者自己的网站地址
    packages=['tool','common'],    #发布的包列表
    platforms='python',
    install_requires=[
        'allure-pytest==2.10.0',
        'configparser==5.2.0',
        'jsonpath==0.82',
        'pytest==7.1.2',
        'requests==2.28.1',
        'xlrd==2.0.1',
    ]  # 安装所需要的库

)